package org.honours;

import java.awt.Point;
import java.io.File;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.swing.SwingUtilities;

import org.expeditee.actions.Actions;
import org.expeditee.gui.*;
import org.expeditee.items.*;

import org.honours.collection.*;
import org.honours.greenstone.ArchiveFileReader;
import org.honours.gui.*;

/**
 * Main class for the cultural collections
 * management system.
 * @author Korii
 */
public class Main {
	
	public static CopyOnWriteArrayList<Collection> _collections = new CopyOnWriteArrayList<Collection>();
	
	public static final String SYSTEM_FRAMESET_NAME = "Honours";
	
	public static final String GSDL_HOME = "greenstone3-svn\\web";
	
	//
	public static final String GSDL_COLLECT_PATH = GSDL_HOME + File.separator + "sites" + File.separator + "localsite" +
			File.separator + "collect" + File.separator;
	
	public static Frame _profile;
	
	//position of assocfilepath annotation on the frame
	public static final int _assocfilePathXpos = 175;
	public static final int _assocfilePathYpos = 101;
	
	/**
	 * Initialize settings and other things for the system.
	 */
	public static void initialize(){
		
		SwingUtilities.invokeLater(new Runnable(){
			
			public void run(){
				loadSettings();
			}
		});
		
	}
	
	/**
	 * Load settings required for the system.
	 */
	private static void loadSettings(){
		
		MessageBay.displayMessage("Initializing System...");
		
		Frame profile = FrameIO.LoadProfile(SYSTEM_FRAMESET_NAME);
		
		if(profile == null){
			
			try{
				profile = FrameIO.CreateNewProfile(SYSTEM_FRAMESET_NAME);
			}catch(Exception e){
				e.printStackTrace();
				MessageBay.errorMessage("ERROR: Unable to load profile.");
				return;
			}
		}
		
		assert(profile != null);
		
		_profile = profile;
		
		if(DisplayIO.getCurrentFrame() != _profile)
			DisplayIO.setCurrentFrame(_profile, true);
		
		//Set up main overlay display
		//Interfaces.setUpOverlayLocations();
		Interfaces.setUpMainOverlay(_profile);
		
		UserSettings.UserName = "honours";
		
		initializeMainFramesets();
		
		addActionsAndAgents();
		addCustomMouseActions();
		addCustomKeyboardActions();
		
		try{
			setUpCollectionObjects();
		}catch(Exception e){
			e.printStackTrace();
			
			MessageBay.errorMessage("ERROR: Unable to load collection objects.");
			return;
		}
		
		MessageBay.displayMessage("System initialized and is now ready for use.");
		
	}
	
	/**
	 * Set up collection objects which last for entire runtime.
	 * A "Collections" directory has been specially created where
	 * collection framesets from GSDL are kept. This method will only
	 * search this directory for collections (and not any other directories
	 * where framesets are kept).
	 * @throws Exception
	 */
	private static void setUpCollectionObjects() throws Exception{
		
		//File collectionsDir = new File(HonoursFrameIO.COLLECTIONS_PATH);
		
		File collectionsDir = new File(FrameIO.FRAME_PATH);
		
		if(collectionsDir.isDirectory()){
			
			//return list of folders in collections directory
			File[] folders = collectionsDir.listFiles();
			
			for(File f : folders){
				
				if(!f.isDirectory())
					continue;
				
				String fName = f.getName();
				
				//TODO: Change this so that "collection" framesets have their own special way of being identified.
				if(fName.equals("collectionsmain") || fName.equals("exhibitionsmain") || fName.equals("interface-overlays") || fName.equals("notesmain") || fName.equals("documentation")){
					continue;
				}
				
				Collection collect = obtainCollectionItems(fName);
				
				if(collect.getItems().size() == 0)
					continue;
				
				_collections.add(collect);
				
			}
			
			for(Collection c : _collections){
				
				for(CollectionItem cItem : c.getItems()){
					
					//Interfaces.setUpOverlayLocations(cItem);
					Interfaces.setUpToolbarOverlay(cItem);
					Interfaces.setUpToggleToolbarOverlay(cItem);
					Interfaces.setUpNotesOverlay(cItem);
					
					HonoursFrameIO.fixTextItemWidths(cItem.getFrame());
				}
			}
			
			MessageBay.displayMessage("Interfaces for collection items initialized.");
		}

	}
	
	/**
	 * This method obtains all frames in a "collection" frameset
	 * and converts them to collection item objects. These items are
	 * then added to a collection object.
	 * @param collectName
	 * @return
	 */
	public static Collection obtainCollectionItems(String collectName){
		
		Collection collect = new Collection(collectName);
		CollectionInformation cInfo = new CollectionInformation();
		
		//Having an issue here when running ImportCollections agent.
		//int last = FrameIO.LoadLast(collectName,HonoursFrameIO.COLLECTIONS_PATH).getNumber();
		int last = FrameIO.getLastNumber(collectName);
		
		//TODO: Add content layering stuff back in once I get just normal
		//loading of collection items working.
		for(int i = 1; i <= last; i++){
			
			//Frame curr = FrameIO.LoadFrame(collectName + i, HonoursFrameIO.COLLECTIONS_PATH);
			Frame curr = FrameIO.LoadFrame(collectName + i);
			
			if(curr == null)
				continue;
			
			String assoc = getAssocFilePath(curr);
			
			if(assoc == null)
				continue;
			
			CollectionItem cItem = new CollectionItem();
			cItem.setFrame(curr);
			HonoursFrameIO.fixTextItemWidths(cItem.getFrame());
			
			cInfo.addRow("I", curr.getName(), assoc);
			
			String docXML = GSDL_COLLECT_PATH + collectName + File.separator + "archives" + File.separator + assoc + File.separator + "doc.xml";
			
			cItem.setDocXML(docXML);
			
			ArchiveFileReader afr = new ArchiveFileReader(docXML);
			
			//System.err.println("*** assoc = " + assoc);
			//System.err.println("*** docXML = " + docXML);
			
			String[] splitAssocFileName = afr.obtainElementText("gsdlassocfile").split(":");
			
			String assocFileName = GSDL_COLLECT_PATH + collectName + File.separator + 
					"index" + File.separator + "assoc" + File.separator + assoc + File.separator + splitAssocFileName[0];
			
			cItem.setAssocfilePath(assoc);
			cItem.setAssociatedFile(assocFileName);
			//setInExpeditee(docXML);
			
			collect.addItem(cItem);
		}
		
		cInfo.sort();
		collect.setCollectionInformation(cInfo);
		return collect;
	}
	
	/**
	 * This method extracts a text annotation on the passed frame
	 * which contains an assocfilepath metadata value 
	 * (e.g. "@assocfilepath HASH0170.dir"). Each frame/collection item
	 * has its own unique assocfilepath value.
	 * TODO: There is a bug in GSDL that is affecting this code here... The 
	 * assocfilepath metadata is being displayed as "HASH0170, HASH0170" for example
	 * when it should only be displayed as "HASH0170.
	 * @param curr - the frame to extract the assocfilepath text annotation from.
	 * @return - the assocfilepath value. If none is found, return null.
	 */
	private static String getAssocFilePath(Frame curr){
		
		String splitString = null;
		
		for(Item item : curr.getItems()){
			if(item instanceof Text){
				Text text = (Text) item;
				
				if(text.getText().startsWith("@assocfilepath")){
					
					String[] split = text.getText().split(" ");
					
					if(splitString == null){
						splitString = split[1];
					
						//remove a comma that may be at the end of the string.
						if(splitString.endsWith(",")){
							splitString = splitString.substring(0, splitString.length()-1);
						}
					}
					
					//do this so that no one can edit/delete 
					//assocfilepath annotation.
					item.setPermission(UserAppliedPermission.none);
					
					//Shift item to position (3,129).
					if(item.getX() != _assocfilePathXpos && item.getY() != _assocfilePathYpos)
						item.setPosition(new Point(_assocfilePathXpos,_assocfilePathYpos));
					
					if(item.getSize() != 14)
						item.setSize(14);
					
					curr.setSaved();
					curr.setChanged(true);
				}
			}
		}
		return splitString;
	}
	
	/**
	 * Initialize (currently three) main framesets:
	 * collections,exhibitions,notes.
	 */
	private static void initializeMainFramesets(){
		
		//NOTE: CollectionsMain not to be confused with the collections
		//directory where all collection framesets are stored.
		String[] mainFramesetNames = new String[]{"CollectionsMain","ExhibitionsMain","NotesMain"};
		
		//TODO: might be better just to check frameset directory
		//instead of trying to load frames...
		for(String s : mainFramesetNames){
			Frame frame = FrameIO.LoadFrame(s + "1");
			
			if(frame == null){
				try{
					FrameIO.CreateNewFrameset(s);
					
					if(s.equals("CollectionsMain")){
						frame = FrameIO.LoadFrame(s + "1");
						frame.addText(329,37,"Import Collections into GlamED", "ImportCollections");
					}
					
				}catch(Exception e){
					e.printStackTrace();
				}
			}else{
				
				if(s.equals("CollectionsMain")){
					//Check frame for an "Import Collections" action text item.
					boolean hit = false;
					int iter = 0;
					
					while(iter < frame.getItems().size() && !hit){
						
						Item currItem = frame.getItems().get(iter);
						if(currItem instanceof Text){
							Text t = (Text)currItem;
							
							if(t.getAction() != null){
								for(String actionString: t.getAction()){
									if(actionString.equals("ImportCollections"))
										hit = true;
								}
							}
						}
						
						iter++;
					}
					
					if(!hit){
						frame.addText(329,37,"Import Collections into GlamED","ImportCollections");
					}
					
				}
			
			}
		}
		
		MessageBay.displayMessage("Main framesets initialized.");
	}
	
	/**
	 * Method to load and set up actions and agents for this system.
	 */
	@SuppressWarnings("rawtypes")
	private static void addActionsAndAgents(){
		
		Actions.Init();
		
		Set<String> agents = new HashSet<String>();
		try{
			for(Class c : Actions.getClasses("org.honours.agents.")){
				if(c != null){
					agents.add(c.getName());
				}else{
					throw new ClassNotFoundException("ERROR: No class found.");
				}
			}
			
		}catch(ClassNotFoundException e){
			e.printStackTrace();
		}
		
		Actions.addAgents(agents);
		
		try{
			for(Class c : Actions.getClasses("org.honours.actions.")){
				
				if(c != null)
					Actions.LoadMethods(c);
				else
					throw new ClassNotFoundException("ERROR: No class found.");
			}
		}catch(ClassNotFoundException e){
			e.printStackTrace();
		}
		
		MessageBay.displayMessage("Actions and agents initialized.");
	}
	
	/**
	 * Method to add custom mouse actions for this system.
	 * These extend on Expedite's mouse actions.
	 */
	private static void addCustomMouseActions(){
		
		MouseActions mouseActions = MouseActions.getInstance();
		
		Browser._theBrowser.getMouseEventRouter().removeExpediteeMouseListener(FrameMouseActions.getInstance());
		Browser._theBrowser.getMouseEventRouter().addExpediteeMouseListener(mouseActions);
		Browser._theBrowser.getMouseEventRouter().removeExpediteeMouseMotionListener(FrameMouseActions.getInstance());
		Browser._theBrowser.getMouseEventRouter().addExpediteeMouseMotionListener(mouseActions);
		
		MessageBay.displayMessage("Mouse actions initialized");
	}
	
	/**
	 * Method to add custom keyboard actions for this system.
	 * These extend on Expeditee's keyboard actions.
	 */
	private static void addCustomKeyboardActions(){
		
		KeyboardActions keyboardActions = KeyboardActions.getInstance();
		Browser._theBrowser.removeKeyListener(FrameKeyboardActions.getInstance());
		Browser._theBrowser.addKeyListener(keyboardActions);
		
		Browser._theBrowser.getContentPane().removeKeyListener(FrameKeyboardActions.getInstance());
		Browser._theBrowser.getContentPane().addKeyListener(keyboardActions);
		
		MessageBay.displayMessage("Keyboard actions initialized");
	}

	/**
	 * Main entry point for this system.
	 * @param args - any arguments passed to this system will be passed to the Expeditee browser.
	 */
	public static void main(final String[] args){
		
		Browser.main(args);
		
		SwingUtilities.invokeLater(new Runnable(){
			public void run(){
				initialize();
			}
		});
	}
	
}